@@ -1,13 +1,16 @@
 #!/usr/bin/env python3
 
+# Portions of this file contributed by NIST are governed by the following
+# statement:
+#
 # This software was developed at the National Institute of Standards
 # and Technology by employees of the Federal Government in the course
-# of their official duties. Pursuant to title 17 Section 105 of the
-# United States Code this software is not subject to copyright
-# protection and is in the public domain. NIST assumes no
-# responsibility whatsoever for its use by other parties, and makes
-# no guarantees, expressed or implied, about its quality,
-# reliability, or any other characteristic.
+# of their official duties. Pursuant to Title 17 Section 105 of the
+# United States Code, this software is not subject to copyright
+# protection within the United States. NIST assumes no responsibility
+# whatsoever for its use by other parties, and makes no guarantees,
+# expressed or implied, about its quality, reliability, or any other
+# characteristic.
 #
 # We would appreciate acknowledgement if the software is used.
 
@@ -19,12 +22,12 @@ This program serves two roles:
 When preparing to build a new monolithic ontology, please edit this variable to match the new respective version.
 """
 
-__version__ = "0.5.0"
+__version__ = "0.6.1"
 
 __all__ = ["CURRENT_CASE_VERSION", "built_version_choices_list"]
 
 # Tested with CI to match versionInfo of <https://ontology.caseontology.org/case/case>.
-CURRENT_CASE_VERSION: str = "1.1.0"
+CURRENT_CASE_VERSION: str = "1.2.0"
 
 # Tested with CI to match set of ontology files available.
 built_version_choices_list = [
@@ -34,6 +37,7 @@ built_version_choices_list = [
     "case-0.7.0",
     "case-0.7.1",
     "case-1.0.0",
+    "case-1.1.0",
     "case-" + CURRENT_CASE_VERSION,
 ]
 
